<?php

namespace passster;

class PS_Dynamic_Styles {
	/**
	 * Contains instance or null
	 *
	 * @var object|null
	 */
	private static $instance = null;

	/**
	 * Returns instance of PS_Dynamic_Styles.
	 *
	 * @return object
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Constructor.
	 *
	 * @return void
	 */
	public function __construct() {
		add_action( 'wp_head', array( $this, 'dynamic_styles' ) );
	}

	/**
	 * Add dynamic styles
	 *
	 * @return void
	 */
	public function dynamic_styles() {
		$options = get_option( 'passster' );

		if ( isset( $options['button_font_size'] ) ) {
			$button_font_size = $options['button_font_size'];
		} else {
			$button_font_size = 12;
		}

		if ( isset( $options['button_font_weight'] ) ) {
			$button_font_weight = $options['button_font_weight'];
		} else {
			$button_font_weight = 400;
		}

		?>
        <style>
            .passster-form {
                max-width: <?php echo esc_html($options['form_max_width']); ?> !important;
            }

            .passster-form > form {
                background: <?php echo esc_html($options['form_background_color']); ?>;
                padding: <?php echo esc_html($options['form_padding']['top']); ?> <?php echo esc_html($options['form_padding']['right']); ?> <?php echo esc_html($options['form_padding']['bottom']); ?> <?php echo esc_html($options['form_padding']['left']); ?>;
                margin: <?php echo esc_html($options['form_margin']['top']); ?> <?php echo esc_html($options['form_margin']['right']); ?> <?php echo esc_html($options['form_margin']['bottom']); ?> <?php echo esc_html($options['form_margin']['left']); ?>;
                border-radius: <?php echo esc_html($options['form_border_radius']); ?>px;
            }

            .passster-form h4 {
                font-size: <?php echo esc_html($options['headline_font_size']); ?>px;
                font-weight: <?php echo esc_html($options['headline_font_weight']); ?>;
                color: <?php echo esc_html($options['headline_font_color']); ?>;
            }

            .passster-form p {
                font-size: <?php echo esc_html($options['instruction_font_size']); ?>px;
                font-weight: <?php echo esc_html($options['instruction_font_weight']); ?>;
                color: <?php echo esc_html($options['instruction_font_color']); ?>;
            }

            .passster-submit, .passster-submit-recaptcha {
                background: <?php echo esc_html($options['button_background_color']); ?>;
                padding: <?php echo esc_html($options['button_padding']['top']); ?> <?php echo esc_html($options['button_padding']['right']); ?> <?php echo esc_html($options['button_padding']['bottom']); ?> <?php echo esc_html($options['button_padding']['left']); ?>;
                margin: <?php echo esc_html($options['button_margin']['top']); ?> <?php echo esc_html($options['button_margin']['right']); ?> <?php echo esc_html($options['button_margin']['bottom']); ?> <?php echo esc_html($options['button_margin']['left']); ?>;
                border-radius: <?php echo esc_html($options['button_border_radius']); ?>px;
                font-size: <?php echo esc_html($button_font_size); ?>px;
                font-weight: <?php echo esc_html($button_font_weight); ?>;
                color: <?php echo esc_html($options['button_font_color']); ?>;
            }

            .passster-submit:hover, .passster-submit-recaptcha:hover {
                background: <?php echo esc_html($options['button_background_color_hover']); ?>;
                color: <?php echo esc_html($options['button_font_color_hover']); ?>;
            }
        </style>
		<?php
	}
}
